/*!
 * autocapture.js v1.0.0-alpha.2.1
 * https://github.com/seeratawan01/autocapture.js#readme
 * Seerat Awan (c) 2022 autocapture.js
 * Released under the MIT License
 */
/**
 * Module provides custom JSON parsing and stringifying functionality.
 * @class JSON
 * @example
 * // To parse a JSON string:
 * JSON.parse('{"key": "value"}')
 * // To stringify a JSON object:
 * JSON.stringify({key: 'value'})
 */ function _arrayLikeToArray$1(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _arrayWithoutHoles$1(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray$1(arr);
}
function _classCallCheck$e(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _iterableToArray$1(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}
function _nonIterableSpread$1() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _toConsumableArray$1(arr) {
    return _arrayWithoutHoles$1(arr) || _iterableToArray$1(arr) || _unsupportedIterableToArray$1(arr) || _nonIterableSpread$1();
}
function _unsupportedIterableToArray$1(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray$1(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$1(o, minLen);
}
var Json = /*#__PURE__*/ function() {
    function Json() {
        _classCallCheck$e(this, Json);
    }
    /**
   * Stringify JSON
   * @param value
   * @param arg
   */ Json.stringify = function stringify(value) {
        for(var _len = arguments.length, arg = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++){
            arg[_key - 1] = arguments[_key];
        }
        var _JSON;
        return (_JSON = JSON).stringify.apply(_JSON, [
            value
        ].concat(_toConsumableArray$1(arg)));
    };
    /**
   * Parse JSON
   * @param value
   */ Json.parse = function parse(value) {
        var parsed = JSON.parse(value);
        if (typeof parsed === "string") parsed = Json.parse(parsed);
        return parsed;
    };
    /**
   * Check if string is valid JSON
   * @param value
   */ Json.isJSON = function isJSON(value) {
        try {
            JSON.parse(value);
        } catch (e) {
            return false;
        }
        return true;
    };
    /**
   * Merge many JSON objects
   */ Json.merge = function merge() {
        for(var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++){
            args[_key] = arguments[_key];
        }
        var result = {};
        for(var i = 0; i < args.length; i++){
            result = Object.assign(result, args[i]);
        }
        return result;
    };
    return Json;
}();

/**
 * Module to provide memory storage functionality.
 * API exposed by the memory storage are made similar to the API of the browser's local storage and session storage to make it easy to switch other persistence methods.
 * @class Store
 * @example
 * const store = Store.getInstance()
 * store.set('key', 'value')
 * store.get('key')
 * store.remove('key')
 * store.clear()
 * store.getAll()
 * store.setAll({key: 'value'})
 * store.has('key')
 * store.size()
 * ...
 */ function _classCallCheck$d(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _defineProperties$1(target, props) {
    for(var i = 0; i < props.length; i++){
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
    }
}
function _createClass$1(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties$1(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties$1(Constructor, staticProps);
    return Constructor;
}
var Store = /*#__PURE__*/ function() {
    function Store() {
        _classCallCheck$d(this, Store);
        this.store = {};
    }
    var _proto = Store.prototype;
    /**
   * Set a key-value pair in the storage.
   * @param key
   * @param value
   */ _proto.setItem = function setItem(key, value) {
        if (Json.isJSON(value)) {
            value = Json.parse(value);
        }
        this.store[key] = value;
    };
    /**
   * Get a value from the storage.
   * @param key
   */ _proto.getItem = function getItem(key) {
        return this.store[key];
    };
    /**
   * Remove a key-value pair from the storage.
   * @param key
   */ _proto.removeItem = function removeItem(key) {
        delete this.store[key];
    };
    /**
   * Clear the storage.
   */ _proto.clear = function clear() {
        this.store = {};
    };
    /**
   * Returns the key at the given index.
   * @param index
   */ _proto.key = function key(index) {
        return Object.keys(this.store)[index];
    };
    /**
   * Returns all the key-value pairs.
   */ _proto.getAll = function getAll() {
        return this.store;
    };
    /**
   * Set all the key-value pairs.
   * @param store
   */ _proto.setAll = function setAll(store) {
        this.store = store;
    };
    /**
   * Returns true if the key exists in the storage.
   * @param key
   */ _proto.has = function has(key) {
        return this.store.hasOwnProperty(key);
    };
    /**
   * Returns the number of key-value pairs.
   */ _proto.size = function size() {
        return Object.keys(this.store).length;
    };
    /**
   * Returns the instance of the Store class.
   * @returns {Store} The instance of the Store class.
   */ Store.getInstance = function getInstance() {
        if (!Store.instance) {
            Store.instance = new Store();
        }
        return Store.instance;
    };
    _createClass$1(Store, [
        {
            key: "length",
            get: /**
   * Returns the number of key/value pairs.
   */ function get() {
                return this.size();
            }
        }
    ]);
    return Store;
}();

// Default values
var DEFAULT_OPTIONS = {
    ELEMENTS: [
        "a",
        "button",
        "form",
        "input",
        "select",
        "textarea",
        "label"
    ],
    ATTRIBUTES: [
        "text",
        "className",
        "value",
        "type",
        "tagName",
        "href",
        "src",
        "id",
        "name",
        "placeholder",
        "title",
        "alt",
        "role"
    ],
    SAFELIST: [],
    CAPTURE: [
        "tap",
        "form",
        "page"
    ],
    PAYLOAD: {},
    PERSISTENCE: "memory",
    STORAGE_KEY: "AUTOCAPTURE_EVENT_DATA",
    VISITOR_ID_KEY: "AUTOCAPTURE_VISITOR_ID",
    MAX_EVENTS: 100,
    MASK_TEXT_CONTENT: false,
    PLUGINS: [
        "scroll",
        "mouse-movement"
    ]
};

function _classCallCheck$c(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _defineProperties(target, props) {
    for(var i = 0; i < props.length; i++){
        var descriptor = props[i];
        descriptor.enumerable = descriptor.enumerable || false;
        descriptor.configurable = true;
        if ("value" in descriptor) descriptor.writable = true;
        Object.defineProperty(target, descriptor.key, descriptor);
    }
}
function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
}
var Persistence = /*#__PURE__*/ function() {
    function Persistence(persistence) {
        _classCallCheck$c(this, Persistence);
        if (persistence === "localStorage" || persistence === "sessionStorage") {
            this.storage = window[persistence];
        } else if (persistence === "memory") {
            this.storage = Store.getInstance();
        } else if (persistence === "none") {
            this.storage = null;
        } else {
            throw new Error("Invalid persistence type");
        }
    }
    var _proto = Persistence.prototype;
    /**
   * Set a key-value pair in the storage.
   */ _proto.setItem = function setItem(key, value) {
        this.storage.setItem(key, value);
    };
    /**
   * Get a value from the storage.
   */ _proto.getItem = function getItem(key) {
        return this.storage.getItem(key);
    };
    /**
   * Remove a key-value pair from the storage.
   */ _proto.removeItem = function removeItem(key) {
        this.storage.removeItem(key);
    };
    /**
   * Clear the storage.
   */ _proto.clear = function clear() {
        this.storage.clear();
    };
    /**
   * Get all the key-value pairs from the storage.
   */ _proto.getAll = function getAll() {
        var store = {};
        for(var i = 0; i < this.storage.length; i++){
            var key = this.storage.key(i);
            store[key] = this.getItem(key);
        }
        return store;
    };
    /**
   * Set all the key-value pairs in the storage.
   */ _proto.setAll = function setAll(store) {
        this.clear();
        for(var key in store){
            this.setItem(key, store[key]);
        }
    };
    /**
   * Check if a key exists in the storage.
   */ _proto.has = function has(key) {
        return this.storage.getItem(key) !== null;
    };
    /**
   * Get the size of the storage.
   */ _proto.size = function size() {
        return this.storage.length;
    };
    /**
   * Get the key at a given index.
   */ _proto.key = function key(index) {
        return this.storage.key(index);
    };
    /**
   * Returns the instance of this class.
   * @param persistence - The type of persistence to use.
   * @returns {Persistence} The instance of this class.
   */ Persistence.getInstance = function getInstance() {
        var persistence = arguments.length > 0 && arguments[0] !== void 0 ? arguments[0] : DEFAULT_OPTIONS.PERSISTENCE;
        if (!Persistence.instance) {
            Persistence.instance = new Persistence(persistence);
        }
        return Persistence.instance.storage;
    };
    _createClass(Persistence, [
        {
            key: "length",
            get: /**
   * Returns the number of key/value pairs.
   */ function get() {
                return this.storage.length;
            }
        }
    ]);
    return Persistence;
}();
/**
   * The storage object.
   * @private
   */ Persistence.instance = null;

/**
 * Module to provide event handling functionality for all DOM Events.
 * With the help of this module, we can capture any event emitted by the browser.
 * @class DOMEvent
 * @example
 * // To capture the click event:
 * const event = new DOMEvent('click')
 * event.capture()
 */ function _classCallCheck$b(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
var DOMEvent = /*#__PURE__*/ function() {
    function DOMEvent(name, handler) {
        var options = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : {}, target = arguments.length > 3 ? arguments[3] : void 0, key = arguments.length > 4 ? arguments[4] : void 0;
        _classCallCheck$b(this, DOMEvent);
        /**
   * the key of the event.
   */ this.key = "dom-event";
        /**
   * Target element to bind an event.
   */ this.target = document;
        this.name = name;
        this.handler = handler;
        this.options = options;
        this.key = key || this.key;
        if (target) this.target = target;
    }
    var _proto = DOMEvent.prototype;
    /**
   * Call the event handler to capture an event.
   * @public
   * @return DOMEvent
   */ _proto.bind = function bind() {
        this.target.addEventListener(this.name, this.handler, this.options);
        return this;
    };
    /**
   * Stop capturing an event.
   * @public
   * @return DOMEvent
   */ _proto.unbind = function unbind() {
        this.target.removeEventListener(this.name, this.handler, this.options);
        return this;
    };
    return DOMEvent;
}();

function _classCallCheck$a(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
var Base = /*#__PURE__*/ function() {
    function Base(settings) {
        _classCallCheck$a(this, Base);
        /**
   * The plugin settings.
   * @protected
   */ this.settings = {};
        /**
   * The persistence object to use for persistence.
   * @protected
   */ this.persistence = null;
        this.settings = settings;
        // Set the persistence object if persistence is not set to none.
        this.persistence = Persistence.getInstance(this.settings.persistence || DEFAULT_OPTIONS.PERSISTENCE, this.settings.maxEvents || DEFAULT_OPTIONS.MAX_EVENTS);
        // On event capture callback
        this.onEventCapture = this.settings.onEventCapture || function(_) {
            return {};
        };
    }
    var _proto = Base.prototype;
    /**
   * Clear all the captured events or specific.
   * @param eventKey - The event to clear.
   * @protected
   */ _proto.clear = function clear(eventKey) {
        if (this.persistence) {
            if (eventKey) {
                this.persistence.removeItem(eventKey);
            } else {
                this.persistence.clear();
            }
        }
    };
    /**
   * Get all the captured events.
   * @protected
   */ _proto.getAll = function getAll() {
        if (this.persistence) {
            return this.persistence.getAll();
        }
        return {};
    };
    return Base;
}();

/**
 * Module to register plugins to extend the functionality of the core.
 * @class PluginRegistry
 * @example
 * // To register a plugin:
 * PluginRegistry.register('scrollMap', scrollMap)
 * // To get a plugin:
 * PluginRegistry.get('scrollMap')
 * // To unregister a plugin:
 * PluginRegistry.unregister('scrollMap')
 *
 */ function _classCallCheck$9(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
var PluginRegistry = /*#__PURE__*/ function() {
    function PluginRegistry() {
        _classCallCheck$9(this, PluginRegistry);
    }
    /**
   * Register a plugin.
   */ PluginRegistry.register = function register(plugin) {
        PluginRegistry.registry.push(plugin);
    };
    /**
   * Unregister a plugin.
   */ PluginRegistry.unregister = function unregister(name) {
        PluginRegistry.registry = PluginRegistry.registry.filter(function(plugin) {
            return plugin.key !== name;
        });
    };
    /**
   * Get a plugin.
   */ PluginRegistry.get = function get(name) {
        return PluginRegistry.registry.find(function(plugin) {
            return plugin.key === name;
        });
    };
    /**
   * Get all plugins.
   */ PluginRegistry.getAll = function getAll() {
        return PluginRegistry.registry;
    };
    /**
   * Call the bind method of a plugin.
   */ PluginRegistry.bind = function bind(name, options) {
        return PluginRegistry.get(name).bind(options);
    };
    /**
   * Check if a plugin is registered.
   */ PluginRegistry.has = function has(name) {
        return !!PluginRegistry.get(name);
    };
    return PluginRegistry;
}();
/**
   * The plugin registry.
   */ PluginRegistry.registry = [];

function _classCallCheck$8(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
var PluginBuilder = /*#__PURE__*/ function() {
    function PluginBuilder() {
        _classCallCheck$8(this, PluginBuilder);
        this.key = "";
        this.options = {};
    }
    var _proto = PluginBuilder.prototype;
    _proto.bind = function bind(_) {
        return [];
    };
    _proto.onBeforeCapture = function onBeforeCapture(_) {
        return true;
    };
    _proto.onEventCapture = function onEventCapture(_) {};
    _proto.onInit = function onInit(options) {
        this.options = options;
    };
    _proto.onStart = function onStart() {};
    _proto.onStop = function onStop() {};
    _proto.getOptions = function getOptions() {
        return this.options;
    };
    return PluginBuilder;
}();

/**
 * This file is used to export all the helper functions that are used in the library to perform some common tasks.
 * The purpose of this file is to keep the code clean and readable.
 * These functions are not exported as part of the library and are only used internally.
 */ function _arrayLikeToArray(arr, len) {
    if (len == null || len > arr.length) len = arr.length;
    for(var i = 0, arr2 = new Array(len); i < len; i++)arr2[i] = arr[i];
    return arr2;
}
function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}
function _instanceof$1(left, right) {
    if (right != null && typeof Symbol !== "undefined" && right[Symbol.hasInstance]) {
        return !!right[Symbol.hasInstance](left);
    } else {
        return left instanceof right;
    }
}
function _iterableToArray(iter) {
    if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter);
}
function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance.\\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}
function _unsupportedIterableToArray(o, minLen) {
    if (!o) return;
    if (typeof o === "string") return _arrayLikeToArray(o, minLen);
    var n = Object.prototype.toString.call(o).slice(8, -1);
    if (n === "Object" && o.constructor) n = o.constructor.name;
    if (n === "Map" || n === "Set") return Array.from(n);
    if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}
/**
 * Get the className of an element, accounting for edge cases where element.className is an object
 * @param {Element} el - element to get the className of
 * @returns {string} the element's class
 */ function getClassName(el) {
    if (_instanceof$1(el.className, SVGAnimatedString)) {
        var ref;
        return ((ref = el.className) === null || ref === void 0 ? void 0 : ref.baseVal) || "";
    }
    return el.className;
}
/**
 * Get the direct text content of an element, accounting for edge cases where element.textContent is an object
 * @param {Element} el - element to get the text of
 * @returns {string} the element's direct text content
 */ function getText(el, masked) {
    var text = "";
    if (_instanceof$1(el.textContent, SVGAnimatedString)) {
        var ref;
        text = ((ref = el.textContent) === null || ref === void 0 ? void 0 : ref.baseVal) || "";
    }
    text = el.textContent;
    if (masked) {
        text = text.replace(/./g, "*");
    }
    return text;
}
/**
 * Get the generated unique id (uuidv4)
 * @returns {string} a unique id
 */ function getUniqueId() {
    return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function(c) {
        var r = Math.random() * 16 | 0, v = c == "x" ? r : r & 0x3 | 0x8;
        return v.toString(16);
    });
}
/**
 * Check whether a DOM event should be "captured" or not.
 * @param elements A list of html tags name to check
 * @param {Event} event - event to check
 * @returns {boolean} whether the event should be captured
 */ function shouldCaptureEvent(elements, event) {
    var target = event.target;
    var tagName = target.tagName.toLowerCase();
    if (elements.indexOf(tagName) === -1) {
        return false;
    }
    // const type = event.type
    //
    // // Form submit events
    // if (type === 'submit' || type === 'input' || type === 'change' || type === 'reset') {
    //   return true
    // }
    //
    // // Mouse events
    // if (type === 'click' || type === 'mousedown' || type === 'mouseup' || type === 'dblclick' || type === 'contextmenu') {
    //   return true
    // }
    //
    // // touch swipe events
    // if (type === 'touchstart' || type === 'touchmove' || type === 'touchend') {
    //   return true
    // }
    return true;
}
/**
 * Get the event target, accounting for edge cases where event.target is an object
 * @param {Event} event - event to get the target of
 * @returns {Element} the event's target
 */ function getEventTarget(event) {
    if (_instanceof$1(event.target, SVGSVGElement)) {
        var ref;
        return (ref = event.target) === null || ref === void 0 ? void 0 : ref.correspondingUseElement;
    }
    return event.target;
}
/**
 * Get the event metadata from user's browser, metadata includes the client specific information
 * @returns {Record<string, any>} the event's metadata
 */ function getEventMetadata() {
    return {
        title: document.title,
        timestamp: Date.now(),
        timezone: new Date().getTimezoneOffset(),
        url: window.location.href,
        userAgent: navigator.userAgent,
        referrer: document.referrer,
        screen: {
            width: window.screen.width,
            height: window.screen.height
        },
        window: {
            width: window.innerWidth,
            height: window.innerHeight
        },
        devicePixelRatio: window.devicePixelRatio,
        language: navigator.language,
        platform: navigator.platform,
        isMobile: isMobileDevice(),
        isTouch: isTouchDevice(),
        isBot: isBot()
    };
}
/**
 * Get the session details from the browser, session details includes the visitor id
 * @param sessionId - provided session id
 * @returns {Record<string, any>} the session details
 */ function getSessionDetails(sessionId) {
    return sessionId ? sessionId : getSessionId();
}
/**
 * Get the session id from the storage
 */ function getSessionId() {
    // Get Storage instance
    var storage = Persistence.getInstance();
    // if no storage is available means persistence is disabled
    if (!storage) {
        return getUniqueId();
    }
    // Get the session id from the storage
    var visitorId = storage.getItem(DEFAULT_OPTIONS.VISITOR_ID_KEY);
    // If the session id is not present in the storage, generate a new one and store it in the storage
    if (!visitorId) {
        var newVisitorId = getUniqueId();
        storage.setItem(DEFAULT_OPTIONS.VISITOR_ID_KEY, newVisitorId);
        return newVisitorId;
    }
    return visitorId;
}
/**
 * Get the data from the captured event according to given attributes and structure it for the API
 * @param {Event} event - event to use
 * @param options - options to use
 * @param options.attributes - attributes to use
 * @param options.sessionId - use custom session id if provided
 * @param options.customPayload - attach custom payload if provided
 * @returns {Object} event data
 */ function prepareEventPayload(event, options) {
    var attributes = options.attributes, sessionId = options.sessionId, payload = options.payload, type = options.type, maskTextContent = options.maskTextContent;
    var target = getEventTarget(event);
    var data = {
        event: type || event.type,
        timestamp: new Date().toISOString(),
        meta: getEventMetadata(),
        session: getSessionDetails(sessionId)
    };
    // Add custom payload if provided not empty object
    if (payload && Object.keys(payload).length > 0) {
        data.payload = payload;
    }
    if (target.tagName) {
        var selector = target.tagName.toLowerCase();
        data.target = {
            selector: selector,
            attributes: {}
        };
        // Add the target's attributes to the payload
        attributes.forEach(function(attribute) {
            var value = getAttributeValue(target, attribute, maskTextContent);
            if (value) {
                data.target.attributes[attribute] = value;
            }
        });
    }
    return data;
}
/**
 * Get the value from given attribute
 * @param {HTMLElement} target - target element
 * @param {string} attribute - attribute name
 * @returns {string} attribute value
 */ function getAttributeValue(target, attribute, masked) {
    switch(attribute){
        case "text":
            return getText(target, masked);
        case "className":
            return getClassName(target);
        case "value":
            if (masked) {
                return target.value && target.value.length > 0 ? "*****" : "";
            }
            return target.value;
        case "type":
            return target.type;
        case "tagName":
            return target.tagName.toLowerCase();
        case "href":
            return target.href;
        case "src":
            return target.src;
        case "id":
            return target.id;
        case "name":
            return target.name;
        case "placeholder":
            return target.placeholder;
        case "title":
            return target.title;
        case "alt":
            return target.alt;
        case "role":
            return target.getAttribute("role");
    }
    // All others attribute
    if (target.hasAttribute(attribute)) {
        return target.getAttribute(attribute);
    }
    return "";
}
/**
 * Method to check if is mobile device
 * @returns {boolean} true if is mobile device
 */ function isMobileDevice() {
    return /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent);
}
/**
 * Method to check if is touch device
 * @returns {boolean} true if is touch device
 */ function isTouchDevice() {
    return "ontouchstart" in window || navigator.maxTouchPoints > 0 || navigator.maxTouchPoints > 0;
}
/**
 * Method to check if is bot
 */ function isBot() {
    return /bot|googlebot|crawler|spider|robot|crawling/i.test(navigator.userAgent);
}
/**
 * Method to store the captured payload in the storage
 * @param {Object} payload - payload to store
 * @returns {boolean} true if the payload is stored successfully
 */ function storePayload(payload, storage) {
    var maxLimit = arguments.length > 2 && arguments[2] !== void 0 ? arguments[2] : DEFAULT_OPTIONS.MAX_EVENTS;
    // if no storage is available means persistence is disabled
    if (!storage) {
        return false;
    }
    // Get the stored payloads
    var storedPayloads = storage.getItem(DEFAULT_OPTIONS.STORAGE_KEY);
    var storedPayloadsArray = storedPayloads && typeof storedPayloads === "string" ? JSON.parse(storedPayloads) : [];
    // if (typeof storedPayloads === 'string') {
    //   storedPayloads = JSON.parse(storedPayloads) || []
    // }
    // If there are no payloads stored, store the current payload
    if (!storedPayloadsArray) {
        storage.setItem(DEFAULT_OPTIONS.STORAGE_KEY, JSON.stringify([
            payload
        ]));
        return true;
    }
    // If the payloads are more than the max limit, remove the oldest one
    if (storedPayloadsArray.length >= maxLimit) {
        storedPayloadsArray.shift();
    }
    // If there are payloads stored, append the current payload to the stored payloads
    storage.setItem(DEFAULT_OPTIONS.STORAGE_KEY, JSON.stringify(_toConsumableArray(storedPayloadsArray).concat([
        payload
    ])));
    return true;
}

function _assertThisInitialized$7(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck$7(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _defineProperty$1(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _getPrototypeOf$7(o) {
    _getPrototypeOf$7 = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf$7(o);
}
function _inherits$7(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf$7(subClass, superClass);
}
function _instanceof(left, right) {
    if (right != null && typeof Symbol !== "undefined" && right[Symbol.hasInstance]) {
        return !!right[Symbol.hasInstance](left);
    } else {
        return left instanceof right;
    }
}
function _objectSpread$1(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty$1(target, key, source[key]);
        });
    }
    return target;
}
function _possibleConstructorReturn$7(self, call) {
    if (call && (_typeof$7(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized$7(self);
}
function _setPrototypeOf$7(o, p) {
    _setPrototypeOf$7 = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf$7(o, p);
}
var _typeof$7 = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct$7() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper$7(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct$7();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf$7(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf$7(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn$7(this, result);
    };
}
/**
 *  A library to provide an easiest and most comprehensive way to automatically capture the user
 *  interactions on your site, from the moment of installation forward. A single snippet grabs
 *  every click, swipe, tap, page-view, and fill — forever.
 *
 *  @class Core
 *  @extends Base
 *  @example
 *  // To use the library:
 *  import AutoCapture from 'autoCapture'
 *  const autoCapture = new AutoCapture({
 *    // options
 *    // ...
 *  })
 *  autoCapture.start()
 *
 */ var AutoCapture = /*#__PURE__*/ function(Base) {
    _inherits$7(AutoCapture, Base);
    var _super = _createSuper$7(AutoCapture);
    function AutoCapture(options) {
        _classCallCheck$7(this, AutoCapture);
        var _this;
        // Set the default options
        var settings = Object.assign({}, {
            elements: DEFAULT_OPTIONS.ELEMENTS,
            attributes: DEFAULT_OPTIONS.ATTRIBUTES,
            safelist: DEFAULT_OPTIONS.SAFELIST,
            capture: DEFAULT_OPTIONS.CAPTURE,
            payload: DEFAULT_OPTIONS.PAYLOAD,
            maskTextContent: DEFAULT_OPTIONS.MASK_TEXT_CONTENT,
            sessionId: "",
            persistence: DEFAULT_OPTIONS.PERSISTENCE,
            maxEvents: DEFAULT_OPTIONS.MAX_EVENTS
        }, options);
        _this = _super.call(this, settings);
        _this.events = [];
        // initialize the plugins
        PluginRegistry.getAll().forEach(function(plugin) {
            plugin.onInit(_objectSpread$1({}, settings));
        });
        return _this;
    }
    var _proto = AutoCapture.prototype;
    /**
   * start capturing the user interactions (click, swipe, tap, page-view, and fill) on the site.
   * @public
   */ _proto.start = function start() {
        // Bind the event listener
        this.bind();
    };
    /**
   * Stop capturing events.
   */ _proto.stop = function stop() {
        // Unbind the event listener
        this.unbind();
        // Stop all the plugins
        PluginRegistry.getAll().forEach(function(plugin) {
            return plugin.onStop();
        });
    };
    /**
   * starting all plugins.
   */ _proto.startPlugins = function startPlugins() {
        var _this = this;
        // start all the plugins
        PluginRegistry.getAll().forEach(function(plugin) {
            // check if the plugin is in capture list
            if (!_this.settings.capture.includes(plugin.key)) {
                return;
            }
            var pluginData = plugin.bind(plugin.getOptions());
            // if the plugin has a bind function, it is not implemented yet
            if (!pluginData) {
                return;
            }
            // loop through the
            pluginData.forEach(function(data) {
                // getting the plugin data
                var target = data.target, event = data.event, callback = data.callback, options = data.options, name = data.name, throttle = data.throttle, condition = data.condition;
                if ((_instanceof(target, HTMLElement) || _instanceof(target, Document) || _instanceof(target, Window)) && typeof callback === "function" && typeof event === "string") {
                    // bind the event
                    _this.events.push(new DOMEvent(event, function(e) {
                        return wrappedHandler(e, name, callback, throttle, condition);
                    }, options, target, plugin.key).bind());
                }
            });
            // wrapping the handler to capture the event
            var wrappedHandler = function(event, type, handler) {
                var throttle = arguments.length > 3 && arguments[3] !== void 0 ? arguments[3] : 0, condition = arguments.length > 4 ? arguments[4] : void 0;
                // if the condition is not met, do not bind the event
                if (condition && typeof condition === "function" && !condition(event)) {
                    return;
                }
                // to prevent massive data collection, we only capture every 100ms
                if (throttle) {
                    var now = Date.now();
                    if (now - _this.lastEvent < throttle) {
                        return;
                    }
                    _this.lastEvent = now;
                }
                if (plugin.onBeforeCapture(event)) {
                    var payload = prepareEventPayload(event, {
                        attributes: _this.settings.attributes,
                        sessionId: _this.settings.sessionId,
                        payload: _this.settings.payload,
                        type: type,
                        maskTextContent: _this.settings.maskTextContent
                    });
                    // adding the data from the handler
                    var data = handler(event, payload);
                    // if the handler returns false, don't capture the event
                    if (data === false) {
                        return;
                    }
                    // if the handler returns an object, merge it with the payload
                    if (typeof data === "object") {
                        payload = Json.merge(payload, {
                            details: data
                        });
                    }
                    if (storePayload(payload, _this.persistence)) {
                        _this.onEventCapture(payload);
                        plugin.onEventCapture(payload);
                    }
                }
            };
        });
    };
    /**
   * Bind the event listener to the elements using the DOMEvent class.
   */ _proto.bind = function bind() {
        // Start all the registered plugins
        this.startPlugins();
    };
    /**
   * Unbind the event listeners.
   */ _proto.unbind = function unbind() {
        this.events.forEach(function(event) {
            return event.unbind();
        });
    };
    /**
   * The function to clear all the captured events from storage.
   */ _proto.clearCapturedEvents = function clearCapturedEvents() {
        this.persistence.clear();
    };
    /**
   * The function to get all the captured events from storage.
   * @public
   * @returns {JSON[]} - An array of JSON objects.
   */ _proto.getCapturedEvents = function getCapturedEvents() {
        var ref;
        var events = (ref = this.persistence) === null || ref === void 0 ? void 0 : ref.getItem(DEFAULT_OPTIONS.STORAGE_KEY);
        if (events) {
            if (typeof events === "string") {
                return Json.parse(events);
            } else {
                return events;
            }
        }
        return [];
    };
    /**
   * Unregister the installed plugin.
   * @public
   * @param {string} key - The plugin key.
   * @returns {void}
   */ _proto.unregisterPlugin = function unregisterPlugin(key) {
        // check if the plugin is installed
        if (!PluginRegistry.has(key)) {
            throw new Error("The plugin ".concat(key, " is not installed."));
        }
        // find the events that are bound to the plugin
        var events = this.events.filter(function(event) {
            return event.key === key;
        });
        // unbind the events
        events.forEach(function(event) {
            return event.unbind();
        });
        // remove the plugin from the registry
        PluginRegistry.unregister(key);
    };
    /**
   * Register the plugin to use it in the AutoCapture instance.
   * @param plugin - function returns an object of type Plugin.
   * @static
   * @public
   */ AutoCapture.use = function use(plugin) {
        // check if the plugin is already registered
        if (PluginRegistry.get(plugin.key)) {
            throw new Error("Plugin with key ".concat(plugin.key, " is already registered."));
        }
        PluginRegistry.register(plugin);
    };
    return AutoCapture;
}(Base);

function _assertThisInitialized$6(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck$6(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _getPrototypeOf$6(o) {
    _getPrototypeOf$6 = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf$6(o);
}
function _inherits$6(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf$6(subClass, superClass);
}
function _possibleConstructorReturn$6(self, call) {
    if (call && (_typeof$6(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized$6(self);
}
function _setPrototypeOf$6(o, p) {
    _setPrototypeOf$6 = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf$6(o, p);
}
var _typeof$6 = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct$6() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper$6(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct$6();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf$6(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf$6(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn$6(this, result);
    };
}
/**
 * Internal scroll plugin to add scroll functionality to the core, functionalities include:
 * - Capture the exact percentage of the page scrolled.
 * - Capture the x and y scroll depth.
 * - Capture the scroll direction (up, down).
 * - Capture the scroll speed.
 * - Capture the scroll position.
 * - Capture the scroll height.
 * @extends PluginBuilder
 */ var ScrollPlugin = /*#__PURE__*/ function(PluginBuilder) {
    _inherits$6(ScrollPlugin, PluginBuilder);
    var _super = _createSuper$6(ScrollPlugin);
    function ScrollPlugin() {
        _classCallCheck$6(this, ScrollPlugin);
        var _this;
        _this = _super.apply(this, arguments);
        _this.key = "scroll";
        _this.lastScrollTop = 0;
        /**
   * A function to get the scroll direction.
   */ _this.getScrollDirection = function() {
            var st = window.pageYOffset || document.documentElement.scrollTop;
            if (st > _this.lastScrollTop) {
                _this.lastScrollTop = st;
                return "down";
            }
            _this.lastScrollTop = st <= 0 ? 0 : st // For Mobile or negative scrolling
            ;
            return "up";
        };
        return _this;
    }
    var _proto = ScrollPlugin.prototype;
    _proto.bind = function bind(_) {
        var _this = this;
        return [
            {
                name: "scroll",
                target: document,
                event: "scroll",
                callback: function() {
                    return _this.captureEvent();
                },
                options: false
            }
        ];
    };
    /**
   * A function to capture the scroll events on your site.
   */ _proto.captureEvent = function captureEvent() {
        // getting the scroll direction
        var direction = this.getScrollDirection();
        // getting the scroll depth
        var scrollDepth = {
            x: window.scrollX,
            y: window.scrollY
        };
        // getting the scroll percentage
        var scrollPercentage = window.scrollY / (document.body.scrollHeight - window.innerHeight) * 100;
        // getting the scroll position
        var scrollPosition = {
            x: window.scrollX / (document.body.scrollWidth - window.innerWidth),
            y: window.scrollY / (document.body.scrollHeight - window.innerHeight)
        };
        // getting the scroll speed
        var scrollSpeed = {
            x: window.scrollX - window.scrollX,
            y: window.scrollY - window.scrollY
        };
        // getting the scroll size
        var scrollSize = {
            x: document.body.scrollWidth - window.innerWidth,
            y: document.body.scrollHeight - window.innerHeight
        };
        // getting the scroll viewport
        var scrollViewport = {
            x: window.innerWidth,
            y: window.innerHeight
        };
        // setting the scroll data
        var payload = {
            direction: direction,
            depth: scrollDepth,
            percentage: scrollPercentage,
            position: scrollPosition,
            speed: scrollSpeed,
            size: scrollSize,
            viewport: scrollViewport
        };
        return payload;
    };
    return ScrollPlugin;
}(PluginBuilder);

/**
 * The plugin to capture the events when the user first visits the page and when the user leaves the page.
 * Scope:
 * - First time page load
 * - When page unload
 * - Browser history change
 * @extends PluginBuilder
 */ function _assertThisInitialized$5(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck$5(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _getPrototypeOf$5(o) {
    _getPrototypeOf$5 = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf$5(o);
}
function _inherits$5(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf$5(subClass, superClass);
}
function _possibleConstructorReturn$5(self, call) {
    if (call && (_typeof$5(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized$5(self);
}
function _setPrototypeOf$5(o, p) {
    _setPrototypeOf$5 = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf$5(o, p);
}
var _typeof$5 = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct$5() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper$5(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct$5();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf$5(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf$5(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn$5(this, result);
    };
}
var PageViewPlugin = /*#__PURE__*/ function(PluginBuilder) {
    _inherits$5(PageViewPlugin, PluginBuilder);
    var _super = _createSuper$5(PageViewPlugin);
    function PageViewPlugin() {
        _classCallCheck$5(this, PageViewPlugin);
        var _this;
        _this = _super.apply(this, arguments);
        _this.key = "page";
        return _this;
    }
    var _proto = PageViewPlugin.prototype;
    _proto.bind = function bind(_) {
        var _this = this;
        return [
            // Detect when the user first visits the page on first load
            {
                name: "page",
                target: window,
                event: "load",
                callback: function() {
                    return _this.captureEvent("page-load");
                },
                options: {
                    once: true,
                    capture: true
                }
            },
            // Detect when the user leaves the page
            {
                name: "page",
                target: window,
                event: "beforeunload",
                callback: function() {
                    return _this.captureEvent("page-leave");
                }
            },
            // Detect when the history state changes
            {
                name: "page",
                target: window,
                event: "popstate",
                callback: function() {
                    return _this.captureEvent("page-change");
                },
                options: {
                    capture: true
                }
            }
        ];
    };
    /**
   * A function to capture the page view events on your site.
   */ _proto.captureEvent = function captureEvent(state) {
        return {
            type: state
        };
    };
    return PageViewPlugin;
}(PluginBuilder);

/**
 * Internal mouse movement plugin to add mouse movement functionality to the core.
 * Scope:
 * - Capture the mouse movement
 * @extends PluginBuilder
 */ function _assertThisInitialized$4(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck$4(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _getPrototypeOf$4(o) {
    _getPrototypeOf$4 = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf$4(o);
}
function _inherits$4(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf$4(subClass, superClass);
}
function _possibleConstructorReturn$4(self, call) {
    if (call && (_typeof$4(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized$4(self);
}
function _setPrototypeOf$4(o, p) {
    _setPrototypeOf$4 = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf$4(o, p);
}
var _typeof$4 = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct$4() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper$4(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct$4();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf$4(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf$4(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn$4(this, result);
    };
}
var MouseMovementPlugin = /*#__PURE__*/ function(PluginBuilder) {
    _inherits$4(MouseMovementPlugin, PluginBuilder);
    var _super = _createSuper$4(MouseMovementPlugin);
    function MouseMovementPlugin() {
        _classCallCheck$4(this, MouseMovementPlugin);
        var _this;
        _this = _super.apply(this, arguments);
        _this.key = "mouse-movement";
        return _this;
    }
    var _proto = MouseMovementPlugin.prototype;
    _proto.bind = function bind(_) {
        var _this = this;
        return [
            {
                name: "mouse-movement",
                target: document,
                event: "mousemove",
                callback: function(event) {
                    return _this.captureEvent(event);
                },
                throttle: 100,
                condition: function() {
                    // Skip the event if the touch is active
                    return !("ontouchstart" in window);
                }
            }
        ];
    };
    /**
   * A function to capture the mouse movement events on your site.
   */ _proto.captureEvent = function captureEvent(event) {
        return {
            x: event.clientX,
            y: event.clientY
        };
    };
    return MouseMovementPlugin;
}(PluginBuilder);

/**
 * Plugin to Detect video play, progress, and complete events as visitors view embedded videos on your site.
 * Scope:
 * - Video state
 * - Video percentage
 * - Video mute
 * - etc...
 * @internal
 * @extends PluginBuilder
 */ function _assertThisInitialized$3(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck$3(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _getPrototypeOf$3(o) {
    _getPrototypeOf$3 = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf$3(o);
}
function _inherits$3(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf$3(subClass, superClass);
}
function _possibleConstructorReturn$3(self, call) {
    if (call && (_typeof$3(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized$3(self);
}
function _setPrototypeOf$3(o, p) {
    _setPrototypeOf$3 = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf$3(o, p);
}
var _typeof$3 = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct$3() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper$3(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct$3();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf$3(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf$3(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn$3(this, result);
    };
}
var VideoPlugin = /*#__PURE__*/ function(PluginBuilder) {
    _inherits$3(VideoPlugin, PluginBuilder);
    var _super = _createSuper$3(VideoPlugin);
    function VideoPlugin() {
        _classCallCheck$3(this, VideoPlugin);
        var _this;
        _this = _super.call(this);
        _this.key = "video";
        return _this;
    }
    var _proto = VideoPlugin.prototype;
    _proto.bind = function bind(_) {
        var _this = this;
        return [
            {
                name: "video",
                target: document.getElementsByTagName("video"),
                event: "load",
                callback: function(event) {
                    return _this.captureEvent(event);
                }
            }
        ];
    };
    /**
   * A function to capture the video events on your site.
   */ _proto.captureEvent = function captureEvent(event) {
        var video = event.target;
        return {
            id: video.id,
            state: video.paused ? "pause" : "play",
            duration: video.duration,
            currentTime: video.currentTime,
            percent: video.currentTime / video.duration,
            volume: video.volume,
            muted: video.muted,
            playbackRate: video.playbackRate,
            buffered: video.buffered,
            ended: video.ended,
            paused: video.paused,
            played: video.played,
            readyState: video.readyState,
            seeking: video.seeking,
            src: video.src,
            videoHeight: video.videoHeight,
            videoWidth: video.videoWidth
        };
    };
    return VideoPlugin;
}(PluginBuilder);

/**
 * Plugin to detect the finger swipe event.
 * Scope:
 * - Detect swipe direction
 * - Detect swipe distance
 * - Detect swipe speed
 * @class SwipePlugin
 * @extends PluginBuilder
 * @experimental
 */ function _assertThisInitialized$2(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck$2(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _getPrototypeOf$2(o) {
    _getPrototypeOf$2 = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf$2(o);
}
function _inherits$2(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf$2(subClass, superClass);
}
function _possibleConstructorReturn$2(self, call) {
    if (call && (_typeof$2(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized$2(self);
}
function _setPrototypeOf$2(o, p) {
    _setPrototypeOf$2 = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf$2(o, p);
}
var _typeof$2 = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct$2() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper$2(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct$2();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf$2(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf$2(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn$2(this, result);
    };
}
var SwipePlugin = /*#__PURE__*/ function(PluginBuilder) {
    _inherits$2(SwipePlugin, PluginBuilder);
    var _super = _createSuper$2(SwipePlugin);
    function SwipePlugin() {
        _classCallCheck$2(this, SwipePlugin);
        var _this;
        _this = _super.apply(this, arguments);
        _this.key = "swipe";
        _this.touchstart = {
            x: 0,
            y: 0
        };
        _this.touchend = {
            x: 0,
            y: 0
        };
        _this.fingerCount = 0;
        _this.minDistance = 50 //Minimum distance for the swipe to work
        ;
        return _this;
    }
    var _proto = SwipePlugin.prototype;
    _proto.bind = function bind() {
        var _this = this;
        return [
            {
                name: "swipe",
                target: document,
                event: "touchstart",
                callback: function(event) {
                    return _this.captureEvent(event);
                }
            },
            {
                name: "swipe",
                target: document,
                event: "touchend",
                callback: function(event) {
                    return _this.captureEvent(event);
                }
            }
        ];
    };
    /**
   * A function to capture the swipe direction, distance and speed.
   * return false if the swipe is less than the minimum distance.
   * return false if the touchstart
   *
   */ _proto.captureEvent = function captureEvent(event) {
        if (event.type === "touchstart") {
            this.touchstart = {
                x: event.touches[0].pageX,
                y: event.touches[0].pageY
            };
            this.fingerCount = event.touches.length;
            return false;
        } else if (event.type === "touchend") {
            this.touchend = {
                x: event.changedTouches[0].pageX,
                y: event.changedTouches[0].pageY
            };
            this.fingerCount = event.touches.length;
        }
        if (this.fingerCount === 1 && this.touchend.x !== 0) {
            var x = this.touchend.x - this.touchstart.x;
            var y = this.touchend.y - this.touchstart.y;
            var distance = Math.sqrt(x * x + y * y);
            var direction = this.getDirection(x, y);
            var speed = this.getSpeed(distance);
            if (distance >= this.minDistance) {
                return {
                    direction: direction,
                    distance: distance,
                    speed: speed
                };
            }
        }
        return false;
    };
    /**
   * A function to get the distance of the swipe.
   * minDistance is the minimum distance for the swipe to work.
   */ /**
   * A function to get the direction (up, down, left right) of the swipe.
   */ _proto.getDirection = function getDirection(x, y) {
        if (Math.abs(x) > Math.abs(y)) {
            return x > 0 ? "right" : "left";
        } else {
            return y > 0 ? "down" : "up";
        }
    };
    /**
   * A function to get the speed of the swipe.
   */ _proto.getSpeed = function getSpeed(distance) {
        return distance / 100;
    };
    return SwipePlugin;
}(PluginBuilder);

/**
 * Plugin to add form functionality to the core.
 * Scope:
 * - Capture the input events
 * - Capture the change events
 * - Capture the form submit events
 * - Capture the form reset events
 * @extends PluginBuilder
 */ function _assertThisInitialized$1(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck$1(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _defineProperty(obj, key, value) {
    if (key in obj) {
        Object.defineProperty(obj, key, {
            value: value,
            enumerable: true,
            configurable: true,
            writable: true
        });
    } else {
        obj[key] = value;
    }
    return obj;
}
function _getPrototypeOf$1(o) {
    _getPrototypeOf$1 = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf$1(o);
}
function _inherits$1(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf$1(subClass, superClass);
}
function _objectSpread(target) {
    for(var i = 1; i < arguments.length; i++){
        var source = arguments[i] != null ? arguments[i] : {};
        var ownKeys = Object.keys(source);
        if (typeof Object.getOwnPropertySymbols === "function") {
            ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function(sym) {
                return Object.getOwnPropertyDescriptor(source, sym).enumerable;
            }));
        }
        ownKeys.forEach(function(key) {
            _defineProperty(target, key, source[key]);
        });
    }
    return target;
}
function _possibleConstructorReturn$1(self, call) {
    if (call && (_typeof$1(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized$1(self);
}
function _setPrototypeOf$1(o, p) {
    _setPrototypeOf$1 = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf$1(o, p);
}
var _typeof$1 = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct$1() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper$1(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct$1();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf$1(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf$1(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn$1(this, result);
    };
}
var FormPlugin = /*#__PURE__*/ function(PluginBuilder) {
    _inherits$1(FormPlugin, PluginBuilder);
    var _super = _createSuper$1(FormPlugin);
    function FormPlugin() {
        _classCallCheck$1(this, FormPlugin);
        var _this;
        _this = _super.apply(this, arguments);
        _this.key = "form";
        return _this;
    }
    var _proto = FormPlugin.prototype;
    _proto.bind = function bind(options) {
        var _this = this;
        return [
            {
                name: "form",
                target: document,
                event: "submit",
                callback: function(event) {
                    return _this.captureEvent(event, options);
                }
            },
            {
                name: "form",
                target: document,
                event: "reset",
                callback: function(event) {
                    return _this.captureEvent(event, options);
                }
            },
            {
                name: "form",
                target: document,
                event: "input",
                callback: function(event) {
                    return _this.captureEvent(event, options);
                }
            },
            {
                name: "form",
                target: document,
                event: "change",
                callback: function(event) {
                    return _this.captureEvent(event, options);
                }
            }
        ];
    };
    /**
   * A function to capture the form events on your site.
   */ _proto.captureEvent = function captureEvent(event, options) {
        var elements = options.elements, safelist = options.safelist;
        // Skip the event if the target is not in the elements list
        if (!shouldCaptureEvent(elements, event)) {
            return false;
        }
        // Skip the event if the target is in the safe list selector
        if (safelist.some(function(selector) {
            return event.target.matches(selector);
        })) {
            return false;
        }
        var target = event.target;
        var formData = {};
        if (target.form) {
            var ref, ref1, ref2, ref3;
            formData = {
                action: (ref = target.form) === null || ref === void 0 ? void 0 : ref.action,
                method: (ref1 = target.form) === null || ref1 === void 0 ? void 0 : ref1.method,
                id: (ref2 = target.form) === null || ref2 === void 0 ? void 0 : ref2.id,
                name: (ref3 = target.form) === null || ref3 === void 0 ? void 0 : ref3.name
            };
        }
        return _objectSpread({
            type: event.type
        }, formData);
    };
    return FormPlugin;
}(PluginBuilder);

/**
 * Plugin to add taps (on touch devices) or clicks (on desktop) to the core.
 * Scope:
 * - Capture the tap/click events
 * @extends PluginBuilder
 */ function _assertThisInitialized(self) {
    if (self === void 0) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
    }
    return self;
}
function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function");
    }
}
function _getPrototypeOf(o) {
    _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
        return o.__proto__ || Object.getPrototypeOf(o);
    };
    return _getPrototypeOf(o);
}
function _inherits(subClass, superClass) {
    if (typeof superClass !== "function" && superClass !== null) {
        throw new TypeError("Super expression must either be null or a function");
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            writable: true,
            configurable: true
        }
    });
    if (superClass) _setPrototypeOf(subClass, superClass);
}
function _possibleConstructorReturn(self, call) {
    if (call && (_typeof(call) === "object" || typeof call === "function")) {
        return call;
    }
    return _assertThisInitialized(self);
}
function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
        o.__proto__ = p;
        return o;
    };
    return _setPrototypeOf(o, p);
}
var _typeof = function(obj) {
    "@swc/helpers - typeof";
    return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj;
};
function _isNativeReflectConstruct() {
    if (typeof Reflect === "undefined" || !Reflect.construct) return false;
    if (Reflect.construct.sham) return false;
    if (typeof Proxy === "function") return true;
    try {
        Boolean.prototype.valueOf.call(Reflect.construct(Boolean, [], function() {}));
        return true;
    } catch (e) {
        return false;
    }
}
function _createSuper(Derived) {
    var hasNativeReflectConstruct = _isNativeReflectConstruct();
    return function _createSuperInternal() {
        var Super = _getPrototypeOf(Derived), result;
        if (hasNativeReflectConstruct) {
            var NewTarget = _getPrototypeOf(this).constructor;
            result = Reflect.construct(Super, arguments, NewTarget);
        } else {
            result = Super.apply(this, arguments);
        }
        return _possibleConstructorReturn(this, result);
    };
}
var TapPlugin = /*#__PURE__*/ function(PluginBuilder) {
    _inherits(TapPlugin, PluginBuilder);
    var _super = _createSuper(TapPlugin);
    function TapPlugin() {
        _classCallCheck(this, TapPlugin);
        var _this;
        _this = _super.apply(this, arguments);
        _this.key = "tap";
        return _this;
    }
    var _proto = TapPlugin.prototype;
    _proto.bind = function bind(options) {
        var _this = this;
        return [
            {
                name: "click",
                target: document,
                event: "click",
                callback: function(event) {
                    return _this.captureEvent(event, options);
                },
                condition: function() {
                    return !("ontouchstart" in window);
                }
            },
            {
                name: "click",
                target: document,
                event: "dblclick",
                callback: function(event) {
                    return _this.captureEvent(event, options);
                },
                condition: function() {
                    return !("ontouchstart" in window);
                }
            },
            {
                name: "click",
                target: document,
                event: "contextmenu",
                callback: function(event) {
                    return _this.captureEvent(event, options);
                }
            },
            {
                name: "tap",
                target: document,
                event: "touchstart",
                callback: function(event) {
                    return _this.captureEvent(event, options);
                }
            }
        ];
    };
    /**
   * A function to capture the tap/click event.
   */ _proto.captureEvent = function captureEvent(event, options) {
        var elements = options.elements, safelist = options.safelist;
        // Skip the event if the target is not in the elements list
        if (!shouldCaptureEvent(elements, event)) {
            return false;
        }
        // Skip the event if the target is in the safe list selector
        if (safelist.some(function(selector) {
            return event.target.matches(selector);
        })) {
            return false;
        }
        // Determine the type of the event.
        var eventType = event.type === "touchstart" ? "tap" : event.type;
        return {
            type: eventType,
            x: event.clientX,
            y: event.clientY
        };
    };
    return TapPlugin;
}(PluginBuilder);

export { DOMEvent, FormPlugin, Json as JSON, MouseMovementPlugin, PageViewPlugin, PluginBuilder, ScrollPlugin, SwipePlugin, TapPlugin, VideoPlugin, AutoCapture as default, prepareEventPayload, shouldCaptureEvent };
//# sourceMappingURL=autocapture.js.map
